@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <cheese-config.h>
+#include "cheese-config.h"
 #endif
 
 #include <string.h>
@@ -34,7 +34,6 @@
 #include <gst/interfaces/xoverlay.h>
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
-#include <glade/glade.h>
 
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
@@ -45,8 +44,7 @@
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
 
-#define GLADE_FILE PACKAGE_DATADIR"/cheese.glade"
-#define UI_FILE PACKAGE_DATADIR"/cheese-ui.xml"
+
 #define SHUTTER_SOUNDS 5
 
 typedef enum
@@ -117,20 +115,6 @@ typedef struct
 
 } CheeseWindow;
 
-static gboolean
-cheese_open_uri (GtkWindow *parent, const char *url, GError **error)
-{
-  gboolean ret;
-  char *cmdline;
-  GdkScreen *screen;
-
-  screen = gtk_window_get_screen (GTK_WINDOW (parent));
-  cmdline = g_strconcat ("xdg-open ", url, NULL);
-  ret = gdk_spawn_command_line_on_screen (screen, cmdline, error);
-  g_free (cmdline);
-
-  return ret;
-}
 
 /* Make url in about dialog clickable */
 static void
@@ -140,7 +124,7 @@ cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointe
   GtkWidget *error_dialog;
   gboolean ret;
 
-  ret = cheese_open_uri (GTK_WINDOW (dialog), url, &error);
+  ret = g_app_info_launch_default_for_uri (url, NULL, &error);
   if (ret == FALSE) {
     error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog), 
                                            GTK_DIALOG_DESTROY_WITH_PARENT, 
@@ -163,7 +147,7 @@ cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpo
 
   uri = g_strconcat ("mailto:", email, NULL);
 
-  ret = cheese_open_uri (GTK_WINDOW (dialog), uri, &error);
+  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
   if (ret == FALSE) {
     error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog), 
                                            GTK_DIALOG_DESTROY_WITH_PARENT, 
@@ -254,7 +238,7 @@ cheese_window_cmd_open (GtkWidget *widget, CheeseWindow *cheese_window)
   uri = g_filename_to_uri (filename, NULL, NULL);
   g_free (filename);
  
-  ret = cheese_open_uri (GTK_WINDOW (cheese_window->window), uri, &error);
+  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
   if (ret == FALSE) {
     dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window), 
                                      GTK_DIALOG_DESTROY_WITH_PARENT, 
@@ -539,7 +523,7 @@ cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
   GError *error = NULL;
   gboolean ret;
 
-  ret = cheese_open_uri (GTK_WINDOW (cheese_window->window), "ghelp:cheese", &error);
+  ret = g_app_info_launch_default_for_uri ("ghelp:cheese", NULL, &error);
 
   if (ret == FALSE) 
   {
@@ -843,8 +827,16 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
 
   if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
   {
-    cheese_countdown_start((CheeseCountdown *) cheese_window->countdown, cheese_window_countdown_picture_cb, cheese_window_countdown_hide_cb, (gpointer) cheese_window);
-    gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook_bar), 1);
+    gboolean countdown;
+    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+    if (countdown) {
+      cheese_countdown_start((CheeseCountdown *) cheese_window->countdown, cheese_window_countdown_picture_cb, cheese_window_countdown_hide_cb, (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook_bar), 1);
+    }
+    else
+    {
+      cheese_window_countdown_picture_cb (cheese_window);
+    }
 
     gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
     // FIXME: set menu inactive
@@ -989,50 +981,60 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
 static void
 cheese_window_create_window (CheeseWindow *cheese_window)
 {
-  GladeXML *gxml;
   GtkWidget *menubar;
   GError *error=NULL;
   char *path;
+  GtkBuilder* builder;
 
-  gxml = glade_xml_new (GLADE_FILE, NULL, NULL);
-  cheese_window->window              = glade_xml_get_widget (gxml, "cheese_window");
-  cheese_window->button_effects      = glade_xml_get_widget (gxml, "button_effects");
-  cheese_window->button_photo        = glade_xml_get_widget (gxml, "button_photo");
-  cheese_window->button_video        = glade_xml_get_widget (gxml, "button_video");
-  cheese_window->image_take_photo    = glade_xml_get_widget (gxml, "image_take_photo");
-  cheese_window->label_effects       = glade_xml_get_widget (gxml, "label_effects");
-  cheese_window->label_photo         = glade_xml_get_widget (gxml, "label_photo");
-  cheese_window->label_take_photo    = glade_xml_get_widget (gxml, "label_take_photo");
-  cheese_window->label_video         = glade_xml_get_widget (gxml, "label_video");
-  cheese_window->main_vbox           = glade_xml_get_widget (gxml, "main_vbox");
-  cheese_window->notebook            = glade_xml_get_widget (gxml, "notebook");
-  cheese_window->notebook_bar        = glade_xml_get_widget (gxml, "notebook_bar");
-  cheese_window->screen              = glade_xml_get_widget (gxml, "video_screen");
-  cheese_window->take_picture        = glade_xml_get_widget (gxml, "take_picture");
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_file (builder, PACKAGE_DATADIR"/cheese.ui", &error);
+
+  if (error)
+  {
+    g_error ("building ui from %s failed: %s", PACKAGE_DATADIR"/cheese.ui", error->message);
+    g_clear_error (&error);
+  }
+
+  cheese_window->window             = GTK_WIDGET (gtk_builder_get_object (builder, "cheese_window"));
+  cheese_window->button_effects     = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects"));
+  cheese_window->button_photo       = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo"));
+  cheese_window->button_video       = GTK_WIDGET (gtk_builder_get_object (builder, "button_video"));
+  cheese_window->image_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "image_take_photo"));
+  cheese_window->label_effects      = GTK_WIDGET (gtk_builder_get_object (builder, "label_effects"));
+  cheese_window->label_photo        = GTK_WIDGET (gtk_builder_get_object (builder, "label_photo"));
+  cheese_window->label_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
+  cheese_window->label_video        = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
+  cheese_window->main_vbox          = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->notebook           = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
+  cheese_window->notebook_bar       = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
+  cheese_window->screen             = GTK_WIDGET (gtk_builder_get_object (builder, "video_screen"));
+  cheese_window->take_picture       = GTK_WIDGET (gtk_builder_get_object (builder, "take_picture"));
+  cheese_window->thumb_scrollwindow = GTK_WIDGET (gtk_builder_get_object (builder, "thumb_scrollwindow"));
+  cheese_window->throbber_frame     = GTK_WIDGET (gtk_builder_get_object (builder, "throbber_frame"));
+  cheese_window->countdown_frame    = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
+  cheese_window->effect_frame       = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
+
+  g_object_unref (builder);
 
   char *str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
   g_free (str);
   gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
 
-  cheese_window->thumb_scrollwindow  = glade_xml_get_widget (gxml, "thumb_scrollwindow");
-  cheese_window->thumb_view          = cheese_thumb_view_new ();
+  cheese_window->thumb_view         = cheese_thumb_view_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_view);
 
   char *gconf_effects;
   g_object_get (cheese_window->gconf, "gconf_prop_selected_effects", &gconf_effects, NULL);
-  cheese_window->effect_frame        = glade_xml_get_widget (gxml, "effect_frame");
   cheese_window->effect_chooser      = cheese_effect_chooser_new (gconf_effects);
   gtk_container_add (GTK_CONTAINER (cheese_window->effect_frame), cheese_window->effect_chooser);
   g_free (gconf_effects);
 
-  cheese_window->throbber_frame      = glade_xml_get_widget (gxml, "throbber_frame");
   cheese_window->throbber            = ephy_spinner_new ();
   ephy_spinner_set_size (EPHY_SPINNER (cheese_window->throbber), GTK_ICON_SIZE_DIALOG);
   gtk_container_add (GTK_CONTAINER (cheese_window->throbber_frame), cheese_window->throbber);
   gtk_widget_show (cheese_window->throbber);
 
-  cheese_window->countdown_frame      = glade_xml_get_widget (gxml, "countdown_frame");
   cheese_window->countdown            = cheese_countdown_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->countdown_frame), cheese_window->countdown);
   gtk_widget_show (cheese_window->countdown);
@@ -1103,11 +1105,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_action_group_set_visible (cheese_window->actions_flickr, path != NULL);
   g_free (path);
 
-  gtk_ui_manager_add_ui_from_file (cheese_window->ui_manager, UI_FILE, &error);
+  gtk_ui_manager_add_ui_from_file (cheese_window->ui_manager, PACKAGE_DATADIR"/cheese-ui.xml", &error);
 
   if (error)
   {
-    g_critical ("building menus from %s failed: %s", UI_FILE, error->message);
+    g_critical ("building menus from %s failed: %s", PACKAGE_DATADIR"/cheese-ui.xml", error->message);
     g_error_free (error);
   }
 
@@ -1162,8 +1164,10 @@ setup_camera (CheeseWindow *cheese_window)
                             cheese_effect_chooser_get_selection (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)));
 
   cheese_webcam_play (cheese_window->webcam);
+  gdk_threads_enter ();
   gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
+  gdk_threads_leave ();
 }
 
 void
